@@ -96,7 +96,8 @@ def group_mean(x, groups, axis=0):
         idxall[axis] = idx
         if idx.sum() > 0:
             norm = 1.0 * (~np.isnan(x[idxall])).sum(axis)
-            xmean[idxall] = np.expand_dims(np.nansum(x[idxall], axis=axis) / norm, axis)
+            ns = np.nansum(x[idxall], axis=axis) / norm
+            xmean[idxall] = np.expand_dims(ns, axis)
             
     return xmean
 
@@ -133,7 +134,8 @@ def group_median(x, groups, axis=0):
         idxall = [slice(None)] * x.ndim
         idxall[axis] = idx
         if idx.sum() > 0:
-            xmedian[idxall] = np.expand_dims(nanmedian(x[idxall], axis=axis), axis)
+            ns = nanmedian(x[idxall], axis=axis)
+            xmedian[idxall] = np.expand_dims(ns, axis)
             
     return xmedian
     
@@ -147,16 +149,11 @@ def unique_group(groups):
     
 # Normalize functions -------------------------------------------------------
 
-def geometric_mean(x, axis=1, check_for_greater_than_zero=True):
+def geometric_mean(x, axis=-1, check_for_greater_than_zero=True):
     """
     Return the geometric mean of matrix x along axis, ignore NaNs.
     
     Raise an exception if any element of x is zero or less.
-    
-    Notes
-    -----
-    The return array has the dimension so it can be broadcasted to 
-    the original array and not the reduced dimension of np.mean
      
     """
     if (x <= 0).any() and check_for_greater_than_zero:
@@ -168,13 +165,17 @@ def geometric_mean(x, axis=1, check_for_greater_than_zero=True):
     m = np.asarray(~m, np.float64)
     m = m.sum(axis)
     x = np.log(x).sum(axis)
-    g = 1.0/m
+    g = 1.0 / m
     x = np.multiply(g, x)
     x = np.exp(x)
     idx = np.ones(x.shape)
-    idx[m == 0] = np.nan
+    if idx.ndim == 0:
+        if m == 0:
+            idx = np.nan
+    else:
+        idx[m == 0] = np.nan
     x = np.multiply(x, idx)
-    return np.expand_dims(x, axis) 
+    return x
 
 def movingsum(x, window, skip=0, axis=-1, norm=False):
     """Moving sum optionally normalized for missing (NaN) data."""
@@ -210,9 +211,9 @@ def movingsum(x, window, skip=0, axis=-1, norm=False):
     initshape[axis] = skip + window - 1
     #Note: skip could be included in starting window
     cutslice = [slice(None)] * x.ndim   
-    cutslice[axis] = slice(None,-skip or None,None)
-    nans = np.nan * np.zeros(initshape)
-    ms = np.concatenate((nans, ms[cutslice]), axis) 
+    cutslice[axis] = slice(None, -skip or None, None)
+    pad = np.nan * np.zeros(initshape)
+    ms = np.concatenate((pad, ms[cutslice]), axis) 
     return ms
 
 def movingsum_forward(x, window, skip=0, axis=-1, norm=False):
@@ -222,7 +223,6 @@ def movingsum_forward(x, window, skip=0, axis=-1, norm=False):
     msf = movingsum(x[flip_index], window, skip=skip, axis=axis, norm=norm)
     return msf[flip_index]
 
-
 def movingrank(x, window, axis=-1):
     """Moving rank (normalized to -1 and 1) of a given window along axis.
 
@@ -236,54 +236,127 @@ def movingrank(x, window, axis=-1):
         raise ValueError, 'Window is too small.'
     nt = x.shape[axis]
     mr = np.nan * np.zeros(x.shape)        
-    for i in xrange(window-1,nt): 
+    for i in xrange(window-1, nt): 
         index1 = [slice(None)] * x.ndim 
         index1[axis] = i
         index2 = [slice(None)] * x.ndim 
         index2[axis] = slice(i-window+1, i+1, None)
-        mr[index1] = np.squeeze(lastrank(x[index2],axis=axis))
-
+        mr[index1] = np.squeeze(lastrank(x[index2], axis=axis))
     return mr
-       
-def lastrank(x, axis=-1):
-    "Rank of last column only"
-    # Note this is just a special case of lastrank_decay with decay=0
-    indlast = [slice(None)] * x.ndim 
-    indlast[axis] = slice(-1, None)
-    indlast2 = [slice(None)] * x.ndim 
-    indlast2[axis] = -1
-    g = (x[indlast] > x).sum(axis)
-    e = (x[indlast] == x).sum(axis)
-    n = np.isfinite(x).sum(axis)
-    r = (g + g + e - 1.0) / 2.0
-    r = r / (n - 1.0)
-    r = 2.0 * (r - 0.5)
-    r[~np.isfinite(x[indlast2])] = np.nan  
-    return np.expand_dims(r,axis)
+   
+def lastrank(x, axis=-1, decay=0.0):
+    """
+    The ranking of the last element along the axis, ignoring NaNs.
+    
+    The ranking is normalized to be between -1 and 1 instead of the more
+    common 1 and N. The results are adjusted for ties. Suitably slicing
+    the output of the `ranking` method will give the same result as
+    `lastrank`. The only difference is that `lastrank` is faster.       
 
-def lastrank_decay(x, decay, axis=-1):
-    "Exponential decay rank of last column only"
-    if decay < 0:
-        raise ValueError, 'decay must be greater than or equal to zero.'
-    nt = x.shape[axis]
-    w = nt - np.ones(nt).cumsum()
-    w = np.exp(-decay * w)
-    w = nt * w / w.sum()
-    matchdim = [None] * x.ndim 
-    matchdim[axis] = slice(None)
-    w = w[matchdim]
+    Parameters
+    ----------
+    x : numpy array
+        The array to rank.
+    axis : int, optional
+        The axis over which to rank. By default (axis=-1) the ranking
+        (and reducing) is performed over the last axis.          
+    decay : scalar, optional
+        Exponential decay strength. Cannot be negative. The default
+        (decay=0) is no decay. In normal ranking (decay=0) all elements
+        used to calculate the rank are equally weighted and so the
+        ordering of all but the last element does not matter. In
+        exponentially decayed ranking the ordering of the elements
+        influences the ranking: elements nearer the last element get more
+        weight.
+        
+    Returns
+    -------
+    d : array
+        In the case of, for example, a 2d array of shape (n, m) and
+        axis=1, the output will contain the rank (normalized to be between
+        -1 and 1 and adjusted for ties) of the the last element of each row.
+        The output in this example will have shape (n,). 
+            
+    Examples
+    -------- 
+    Create an array:
+                    
+    >>> y1 = larry([1, 2, 3])
+    
+    What is the rank of the last element (the value 3 in this example)?
+    It is the largest element so the rank is 1.0:
+    
+    >>> import numpy as np
+    >>> from la.afunc import lastrank
+    >>> x1 = np.array([1, 2, 3])
+    >>> lastrank(x1)
+    1.0
+    
+    Now let's try an example where the last element has the smallest
+    value:
+    
+    >>> x2 = np.array([3, 2, 1])
+    >>> lastrank(x2)
+    -1.0
+    
+    Here's an example where the last element is not the minimum or maximum
+    value:
+    
+    >>> x3 = np.array([1, 3, 4, 5, 2])
+    >>> lastrank(x3)
+    -0.5
+    
+    Finally, let's add a large decay. The decay means that the elements
+    closest to the last element receive the most weight. Because the
+    decay is large, the first element (the value 1) doesn't get any weight
+    and therefore the last element (2) becomes the smallest element:
+    
+    >>> lastrank(x3, decay=10)
+    -1.0
+    
+    """
+    if x.size == 0:
+        # At least one dimension has length 0
+        shape = list(x.shape)
+        shape.pop(axis)
+        r = nans(shape, dtype=x.dtype) 
+        if (r.ndim == 0) and (r.size == 1):
+            r = np.nan     
+        return r 
     indlast = [slice(None)] * x.ndim 
     indlast[axis] = slice(-1, None)
     indlast2 = [slice(None)] * x.ndim 
-    indlast2[axis] = -1
-    g = ((x[indlast] > x) * w).sum(axis)
-    e = ((x[indlast] == x) * w).sum(axis)
-    n = (np.isfinite(x) * w).sum(axis)
-    r = (g + g + e - w.flat[-1]) / 2.0
-    r = r / (n - w.flat[-1])
-    r = 2.0 * (r - 0.5)
-    r[~np.isfinite(x[indlast2])] = np.nan
-    return np.expand_dims(r, axis)
+    indlast2[axis] = -1  
+    if decay > 0:
+        # Exponential decay 
+        nt = x.shape[axis]
+        w = nt - np.ones(nt).cumsum()
+        w = np.exp(-decay * w)
+        w = nt * w / w.sum()
+        matchdim = [None] * x.ndim 
+        matchdim[axis] = slice(None)
+        w = w[matchdim]
+        g = ((x[indlast] > x) * w).sum(axis)
+        e = ((x[indlast] == x) * w).sum(axis)
+        n = (np.isfinite(x) * w).sum(axis)
+        r = (g + g + e - w.flat[-1]) / 2.0
+        r = r / (n - w.flat[-1])
+    elif decay < 0:
+        raise ValueError, 'decay must be greater than or equal to zero.'        
+    else:
+        # Special case the most common case, decay = 0, for speed
+        g = (x[indlast] > x).sum(axis)
+        e = (x[indlast] == x).sum(axis)
+        n = np.isfinite(x).sum(axis)
+        r = (g + g + e - 1.0) / 2.0
+        r = r / (n - 1.0)      
+    r = 2.0 * (r - 0.5)    
+    if x.ndim == 1:
+        if not np.isfinite(x[indlast2]):
+            r = np.nan
+    else:
+        r[~np.isfinite(x[indlast2])] = np.nan
+    return r    
 
 def ranking(x, axis=0, norm='-1,1', ties=True):
     """
@@ -295,11 +368,14 @@ def ranking(x, axis=0, norm='-1,1', ties=True):
         Data to be ranked.
     axis : int, optional
         Axis to rank over. Default axis is 0.
-    norm: str
+    norm: str, optional
         A string that specifies the normalization:
-        '0,N-1'     Zero to N-1 ranking
-        '-1,1'      Scale zero to N-1 ranking to be between -1 and 1
-        'gaussian'  Rank data then scale to a Gaussian distribution
+            ==========  ================================================
+            '0,N-1'     Zero to N-1 ranking
+            '-1,1'      Scale zero to N-1 ranking to be between -1 and 1
+            'gaussian'  Rank data then scale to a Gaussian distribution
+            ==========  ================================================
+        The default ranking is '-1,1'.
     ties: bool
         If two elements of `x` have the same value then they will be ranked
         by their order in the array (False). If `ties` is set to True
@@ -378,7 +454,9 @@ def push(x, n, axis=-1):
     "Fill missing values (NaN) with most recent non-missing values if recent."
     if axis != -1 or axis != x.ndim-1:
         x = np.rollaxis(x, axis, x.ndim)
-    y = np.array(x)       
+    y = np.array(x) 
+    if y.ndim == 1:
+        y = y[None, :]
     fidx = np.isfinite(y)
     recent = np.nan * np.ones(y.shape[:-1])  
     count = np.nan * np.ones(y.shape[:-1])          
@@ -389,9 +467,11 @@ def push(x, n, axis=-1):
         y[idx, i] = recent[idx]
         idx = fidx[...,i]
         count[idx] = i
-        recent[idx] = y[idx,i]
+        recent[idx] = y[idx, i]
     if axis != -1 or axis != x.ndim-1:
         y = np.rollaxis(y, x.ndim-1, axis)
+    if x.ndim == 1:
+        return y[0]
     return y
 
 def _quantileraw1d(xi, q):
@@ -402,10 +482,10 @@ def _quantileraw1d(xi, q):
     if nx:
         jdx = xi.argsort(axis=0).argsort(axis=0)
         mdx = np.nan * jdx
-        kdx = 1.0 * (nx - 1) / (q) * np.ones((q,1))
+        kdx = 1.0 * (nx - 1) / (q) * np.ones((q, 1))
         kdx = kdx.cumsum(axis=0)
-        kdx = np.concatenate((-1*np.ones((1,kdx.shape[1])), kdx), 0)
-        kdx[-1,0] = nx
+        kdx = np.concatenate((-1 * np.ones((1, kdx.shape[1])), kdx), 0)
+        kdx[-1, 0] = nx
         for j in xrange(1, q+1):
             mdx[(jdx > kdx[j-1]) & (jdx <= kdx[j]),:] = j
         y[idx] = mdx
@@ -471,7 +551,7 @@ def covMissing(R):
 
 # Random functions ----------------------------------------------------------
 
-def shuffle(x, axis=0, rs=None):
+def shuffle(x, axis=0):
     """
     Shuffle the data inplace along the specified axis.
     
@@ -496,8 +576,55 @@ def shuffle(x, axis=0, rs=None):
 # NaN functions -------------------------------------------------------------
 
 def nans(shape, dtype=float):
-    "Works like ones and zeros except that the fill value is NaN"
+    """
+    Works like ones and zeros except that the fill value is NaN (by default)
+    
+    The fill value is determined by dtype. The fill value is NaN for floats,
+    '' for str, and None for object. A TypeError is raised for int dtype.
+    
+    Parameters
+    ----------
+    shape : {tuple, int}
+        The desired shape pf the output
+    dtype : various
+        The desired dtype of output. Typically values are float, str, object.
+        Integer dtype is not allowed.
+        
+    Returns
+    -------
+    a : ndarray
+        An array of shape `shape` and dtype `dtype`. The fill value is NaN
+        for floats, '' for str, and None for object.
+        
+    Examples
+    --------            
+    >>> nans(2)
+    array([ NaN,  NaN])
+    >>> nans((2,2))
+    array([[ NaN,  NaN],
+           [ NaN,  NaN]])
+    >>> nans(2, str)
+    array(['', ''], 
+          dtype='|S1')
+    >>> nans(2, object)
+    array([None, None], dtype=object)
+    >>> import datetime
+    >>> nans(2, datetime.date)
+    array([None, None], dtype=object)
+    
+    """
     a = np.empty(shape, dtype)
-    a.fill(np.nan)
+    # Check for str first since numpy considers str to be a scalar subtype.
+    # No check for object dtype since empty already fills those with None
+    if np.issubdtype(a.dtype, str):
+        a.fill('')      
+    elif np.issctype(a.dtype):
+        if issubclass(a.dtype.type, np.inexact):
+            a.fill(np.nan)
+        else:
+            # int dtype can't be filled with NaN
+            msg = 'Inexact scalar dtype, such as float, needed for NaN '
+            msg += 'fill value.'
+            raise TypeError, msg      
     return a
 
